/**
 *	>>> ASpongy <<< - by JAC! on 2013-03-30 
 * 
 *	The compo filler I always wanted to release.
 */
package com.wudsn.productions.atari800.aspongy;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.Panel;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.util.Map;
import java.util.Random;
import java.util.TreeMap;

import javax.swing.ImageIcon;
import javax.swing.JLabel;

public abstract class ASpongy {

    /**
     * Statistic utility class.
     */
    private static final class Variable {
	private int minValue = Integer.MAX_VALUE;
	private int maxValue = Integer.MIN_VALUE;

	public Variable(int value) {
	    setValue(value);
	}

	public void setValue(int value) {
	    if (value < minValue) {
		minValue = value;
	    }
	    if (value > maxValue) {
		maxValue = value;
	    }
	}

	public int getSpan() {
	    return maxValue - minValue;
	}

	private static int getBits(int value) {
	    value = Math.abs(value);
	    return 32 - Integer.numberOfLeadingZeros(value);
	}

	@Override
	public String toString() {
	    return "min=" + minValue + "(" + getBits(minValue) + ") max=" + maxValue + "(" + getBits(maxValue)
		    + ") span=" + getSpan() + "(" + getBits(getSpan()) + ")" + "\n";
	}
    }

    public static final int XSIZE = 96;
    public static final int YSIZE = XSIZE;

    private Container container;
    private BufferedImage bufferedImage;
    private AffineTransform tx;
    private JLabel imageLabel;
    private JLabel textLabel;

    private int[] palette;
    private Random paletteRandom;
    private boolean[] paletteRandomArray;

    protected static final short B127 = 127;
    protected static final short const64 = 64;
    protected int[] N;

    protected short[] mfactors;

    private boolean recordVariables;
    private Map<String, Variable> variableValues;

    protected ASpongy() {
	final int ZOOM = 4;
	bufferedImage = new BufferedImage(XSIZE * ZOOM, YSIZE * ZOOM, BufferedImage.TYPE_INT_RGB);

	tx = new AffineTransform();
	tx.setToScale(4, 4);

	ImageIcon imageIcon;

	imageIcon = new ImageIcon();
	imageIcon.setImage(bufferedImage);
	imageLabel = new JLabel();
	imageLabel.setIcon(imageIcon);
	imageLabel.setSize(XSIZE * ZOOM, YSIZE * ZOOM);

	textLabel = new JLabel();
	textLabel.setText("0");

	container = new Panel(new BorderLayout());
	container.add(BorderLayout.CENTER, imageLabel);
	container.add(BorderLayout.SOUTH, textLabel);

	N = new int[128];
	for (int i = 127; i-- > 0;) {
	    N[i] = (int) (1.0f * i / 43) & 1;
	}

	mfactors = new short[] { 1, 3, 9, 27, 81 };

	variableValues = new TreeMap<String, Variable>();
    }

    public final void setRecordVariables(boolean recordVariables) {
	this.recordVariables = recordVariables;
    }

    public final Container getContainer() {
	return container;
    }

    public final void setPalette(int[] palette) {
	if (palette == null) {
	    throw new IllegalArgumentException("Parameter 'palette' must not be null.");
	}
	this.palette = palette;
	paletteRandom = new Random();
	paletteRandomArray = new boolean[256];
	for (int i = 0; i < 255; i++) {
	    paletteRandomArray[i] = (paletteRandom.nextDouble() < i / 256.0d - .25);
	}
	int i = 1;
	i = i + 1;
    }

    /**
     * To be redefined.
     */
    public void init() {
    }

    public abstract void frame(float a, float b, int t, int[] imageData);

    protected final void println(String string) {
	if (string == null) {
	    throw new IllegalArgumentException("Parameter 'string' must not be null.");
	}
	System.out.println(getClass().getName() + ":" + string);

    }

    public final void paint(int[] imageData, long duration, boolean dither) {
	int index = 0;
	Random random = new Random();
	dither = false;

	for (int py = 0; py < YSIZE; py++) {
	    for (int px = 0; px < XSIZE; px++) {
		int offset = imageData[index++];
		if (dither) {
		    offset += -5 + random.nextInt(10);

		    if (offset < 0) {
			offset = 0;
		    }
		    if (offset > 255) {
			offset = 255;
		    }
		}
		int color;
		if (palette.length == 2) {
		    color = paletteRandomArray[offset] ? palette[0] : palette[1];
		    // color =144 > offset ? palette[0]: palette[1];
		} else {
		    color = palette[offset];
		}
		bufferedImage.setRGB(px, py, color);
	    }
	}

	Graphics2D g2d = (Graphics2D) imageLabel.getGraphics();
	g2d.drawImage(bufferedImage, tx, null);
	textLabel.setForeground(Color.BLACK);
	textLabel.setText(duration / 100000 + "* 100 s");

    }

    public final void setCurrentMinimumIndicator() {
	textLabel.setForeground(Color.GREEN);

    }

    protected final void recordVariable(String name, int value) {
	if (name == null) {
	    throw new IllegalArgumentException("Parameter 'name' must not be null.");
	}
	if (recordVariables) {
	    Variable variable = variableValues.get(name);
	    if (variable == null) {
		variableValues.put(name, new Variable(value));
	    } else {
		variable.setValue(value);
	    }
	}
    }

    public final void printVariables() {
	println(variableValues.toString());
    }
}