package com.wudsn.productions.atari800.aspongy;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;


/*
 {
 X=min=-2899508(22) max=3901125(22) span=6800633(23)
 , XS=min=1048576(21) max=1048576(21) span=0(0)
 , Y=min=-2714792(22) max=2953822(22) span=5668614(23)
 , YS=min=1048576(21) max=1048576(21) span=0(0)
 , Z=min=-1957234(21) max=4529900(23) span=6487134(23)
 , ZS=min=0(0) max=2080768(21) span=2080768(21)
 , u=min=-31448(15) max=24685(15) span=56133(16)
 , u0=min=-24791(15) max=24791(15) span=49582(16)
 , v=min=-31449(15) max=19480(15) span=50929(16)
 , w=min=-24484(15) max=24484(15) span=48968(16)
 , ybcos=min=-16384(15) max=16383(14) span=32767(15)
 , ybsin=min=-16383(14) max=16383(14) span=32766(15)
 }

 */
public final class ASpongy5Short extends ASpongy {

    private static final int BASE_SHIFT = 14;
    private static final int BASE = 1 << BASE_SHIFT;

    // private static final int BASE_SQRT = 128;
    // private static final int BASE_SQRT_SHIFT = BASE_SHIFT / 2;

    public static void main(String[] args) throws IOException {
	generateSinus(1);
	generateSinus(96 / 2);
	generateSinus(64 / 2);
    }

    private static void generateSinus(int divisor)
	    throws FileNotFoundException, IOException {
	int maxTime = 128;
	byte[] sinus = new byte[2 * maxTime];
	for (int i = 0; i < maxTime; i++) {
	    float a = (float) (2.0f * Math.PI * i / maxTime);
	    short cos16 = (short) (cos16(a) / divisor);
	    sinus[i] = (byte) (cos16 & 0xff);
	    sinus[i + maxTime] = (byte) (cos16 >> 8);
	}
	File file = new File("ASpongy-" + divisor + ".sin");
	FileOutputStream fos = new FileOutputStream(file);
	fos.write(sinus);
	fos.close();
	System.out.println("Output file '" + file.getAbsolutePath()
		+ "' created.");
    }

    private static short cos16(float a) {
	return (short) (Math.cos(a) * BASE);
    }

    private static short mult16(short a, short b) {
	long temp = (a * b) >> BASE_SHIFT;
	return (short) temp;
    }

    @Override
    public void frame(float a, float b, int t, int[] imageData) {

	short acos, asin;
	short bcos, bsin;

	acos = cos16(a);
	asin = cos16(a + 8);

	bcos = cos16(b);
	bsin = cos16(b + 8);

	recordVariable("bcos", bcos);
	recordVariable("bsin", bsin);

	short imageIndex = 0;

	int XS, YS, ZS;
	XS = const64 << BASE_SHIFT;
	YS = const64 << BASE_SHIFT;
	ZS = ((t & 127) << BASE_SHIFT); // Move through

	short ybsin = (short) -bsin;
	short ybcos = (short) -bcos;

	recordVariable("XS", XS);
	recordVariable("YS", YS);
	recordVariable("ZS", ZS);

	// System.out.println("XS="+Integer.toBinaryString(XS)
	// +" YS="+Integer.toBinaryString(YS)+" ZS="+Integer.toBinaryString(ZS));

	// println("ybsin=" + (ybsin * 1.0d) / BASE + " ybcos=" + (ybcos *
	// 1.0d)/ BASE);
	for (int y = 0; y < YSIZE; y++) {
	    recordVariable("ybsin", ybsin);
	    recordVariable("ybcos", ybcos);

	    int h = 0;

	    short u0, u, v, w;
	    u0 = (short) (asin - acos);
	    w = (short) (asin + acos);

	    for (int x = 0; x < XSIZE; x++, imageData[imageIndex++] = h << 1) {
		recordVariable("u0", u0);
		recordVariable("w", w);
		final short accuracy = 1;
		final short accuracy2 = accuracy * accuracy;
		// http://www.6502.org/source/integers/fastmult.htm
		v = (short) (ybcos - (mult16((short) (u0 / accuracy),
			(short) (bsin / accuracy)) * accuracy2));
		u = (short) (ybsin + (mult16((short) (u0 / accuracy),
			(short) (bcos / accuracy)) * accuracy2));

		recordVariable("u", u);
		recordVariable("v", v);

		h = 127;

		int X, Y, Z;
		X = XS;
		Y = YS;
		Z = ZS;

		boolean done = false;
		while (h > 0 && !done) {
		    recordVariable("X", X);
		    recordVariable("Y", Y);
		    recordVariable("Z", Z);

		    int XM, YM, ZM;
		    XM = X;
		    YM = Y;
		    ZM = Z;

		    for (int i = 1; i < 5; i++) {

			int sum;
			int pos =(XM >> BASE_SHIFT);
			sum = N[(pos) & B127]
				+ N[(YM >> BASE_SHIFT) & B127]
				+ N[(ZM >> BASE_SHIFT) & B127];

			if (sum >= 2) {
			    break;
			}

			// { 1, 3, 9, 27, 81 };
			if (i == 4) {
			    done = true;
			} else {
			    XM = XM + XM + XM;
			    YM = YM + YM + YM;
			    ZM = ZM + ZM + ZM;
			}
		    }
		    X += u;
		    Y += v;
		    Z += w;
		    h--;
		}

		u0 = (short) (u0 + (2 * acos / XSIZE));
		w = (short) (w - (2 * asin / XSIZE));
	    }

	    ybsin = (short) (ybsin + (2 * bsin / YSIZE));
	    ybcos = (short) (ybcos + (2 * bcos / YSIZE));
	}

    }

}
