//
//	>>> R.G.B.A. - Rasters Girls Beer Atari by JAC! <<<
//
//	Created for Sillyventure 2012 in Gdansk.
//
//	Utility class for RGBA-Rotator and RGBA-Twirl

package com.wudsn.demos.atari800.rgba;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;

import javax.swing.JApplet;

@SuppressWarnings("serial")
public class RGBARotator extends JApplet {

    private static final int XSIZE = 42;
    private static final int YSIZE = 25;
    private static final int ZOOM = 8;

    private BufferedImage bi;

    public RGBARotator() {

    }

    @Override
    public void init() {

	setSize((XSIZE * 2) * ZOOM * 2 + 32, (YSIZE * 2) * ZOOM + 32);
	bi = new BufferedImage(XSIZE * 2, YSIZE * 2, BufferedImage.TYPE_INT_RGB);
	createLuma(bi, 41, 25, 21, 13, true, true);
	createDeg(bi, 40, 25, 40, 13, true, false);
    }

    private void print(Object o) {
	System.out.print(o);
    }

    private void println() {
	System.out.println();
    }

    private void println(Object o) {
	System.out.println(o);
    }

    @Override
    public void paint(Graphics g) {
	Graphics2D g2d = (Graphics2D) g;
	AffineTransform tx = new AffineTransform();
	tx.setToScale(ZOOM, ZOOM);
	g2d.drawImage(bi, tx, this);
    }

    private void printArray(String name, List<Integer> array, int width,
	    boolean hex) {
	if (name == null) {
	    throw new IllegalArgumentException(
		    "Parameter 'name' must not be null.");
	}
	if (array == null) {
	    throw new IllegalArgumentException(
		    "Parameter 'array' must not be null.");
	}
	print("\t.local " + name + " ;" + array.size() + " bytes");
	for (int i = 0; i < array.size(); i++) {
	    if ((i % width) == 0) {
		print("\n\t" + ".byte ");
	    }
	    int value1 = array.get(i).intValue();
	    String stringValue;
	    if (hex) {
		stringValue = (value1 < 16 ? "$0" : "$")
			+ Integer.toHexString(value1);
	    } else {
		stringValue = Integer.toString(value1);
	    }
	    print(stringValue);
	    if ((i % width) < width - 1 && i < array.size() - 1) {
		print(",");
	    }
	}
	print("\n\t.endl\n\n");
    }

    private void count(int keyInt, TreeMap<Integer, Integer> map) {
	Integer key = new Integer(keyInt);
	Integer value1 = map.get(key);
	if (value1 == null) {
	    map.put(key, new Integer(1));
	} else {
	    map.put(key, new Integer(value1.intValue() + 1));

	}

    }

    private void createLuma(BufferedImage bi, int xsize, int ysize, int xmax,
	    int ymax, boolean sectionOnly, boolean diagonalOnly) {
	if (bi == null) {
	    throw new IllegalArgumentException(
		    "Parameter 'bi' must not be null.");
	}

	List<Integer> indexArray = new ArrayList<Integer>();

	int lumMax = -1;
	int lumMin = 256;
	TreeMap<Integer, Integer> lumMap = new TreeMap<Integer, Integer>();
	List<Integer> lumArray = new ArrayList<Integer>();

	int side_max = Math.max(xsize, ysize);
	double side_div = 1.0d * (side_max - 1);
	double x_lenght = 1;
	double y_lenght = (xmax - ymax) * 1.0 / (ymax);
	double norm = Math.sqrt(x_lenght * x_lenght + y_lenght * y_lenght);
	int size = 0;
	boolean indexRequired = true;
	for (int y = 0; y < side_max; y++) {
	    if (indexRequired) {
		indexArray.add(new Integer(size));
		indexRequired = false;
	    }
	    for (int x = 0; x < side_max; x++) {
		double py = 2 * y / side_div - 1.0d;
		double px = 2 * x / side_div - 1.0d;

		double value0, value1;

		value0 = Math.sqrt(px * px + py * py);
		value1 = value0 / norm;

		if (!sectionOnly
			|| (x < xmax && y > (xmax - ymax - 1) && y < xmax)) {
		    if (!diagonalOnly || y >= x) {

			int lum = (int) (256 * value1);
			lum=lum/4;
			int col = 0x010101 * lum;
			bi.setRGB(x, y, col);

			lum = lum / 1; // Normalize lum
			if (lum > lumMax) {
			    lumMax = lum;
			}
			if (lum < lumMin) {
			    lumMin = lum;
			}
			count(lum, lumMap);

			println("LUMA: x=" + x + " y=" + y + " px=" + px
				+ " py=" + py + " value0=" + value0
				+ " value1=" + value1 + " lum=" + lum);
			lumArray.add(new Integer(lum));
			size++;
			indexRequired = true;
		    }
		}
	    }

	}
	indexArray.remove(indexArray.size() - 1);

	println("totalRecordCount=" + size);
	println("lumMin=" + lumMin + " lumMax=" + lumMax + " totalRecordCount="
		+ lumMap.size());
	println(lumMap.toString());

	printArray("lumindextab", indexArray, indexArray.size(), false);
	printArray("lumtab", lumArray, xmax, true);
    }

    private void createDeg(BufferedImage b, int xsize, int ysize, int xmax,
	    int ymax, boolean sectionOnly, boolean diagonalOnly) {
	if (bi == null) {
	    throw new IllegalArgumentException(
		    "Parameter 'bi' must not be null.");
	}

	List<Integer> indexArray = new ArrayList<Integer>();

	int degMax = -1;
	int degMin = 256;
	TreeMap<Integer, Integer> degMap = new TreeMap<Integer, Integer>();
	List<Integer> degArray = new ArrayList<Integer>();

	double xside_div = 1.0d * (xsize);
	double yside_div = 1.0d * (ysize );
	int size = 0;
	boolean indexRequired = true;
	for (int y = 0; y < ysize; y++) {
	    if (indexRequired) {
		indexArray.add(new Integer(size));
		indexRequired = false;
	    }
	    for (int x = 0; x < xsize ; x++) {
		double py = 2 * y / yside_div - 1.0d;
		double px = 2 * x / xside_div - 1.0d;

		if (!sectionOnly || y < ymax) {
		    if (!diagonalOnly || y >= x) {

			double value;
			value = Math.atan2(px, -py);

			int deg = ((int) (value / Math.PI * 128.f) - 192);
			deg &= 0xff;
			int col = 0x010101 * deg;

			deg = deg / 4; // Normalize deg
			if (deg > degMax) {
			    degMax = deg;
			}
			if (deg < degMin) {
			    degMin = deg;
			}
			count(deg, degMap);

			bi.setRGB(x + XSIZE, y, col);
			println("DEG: x=" + x + " y=" + y + " px=" + px
				+ " py=" + py + " value=" + value + " deg="
				+ deg);
			degArray.add(new Integer(deg));
			size++;
			indexRequired = true;
		    }
		}
	    }

	}
	indexArray.remove(indexArray.size() - 1);

	println("totalRecordCount=" + indexArray.size());

	println("degMin=" + degMin + " degMax=" + degMax + " totalRecordCount="
		+ degMap.size());
	println(degMap.toString());

	printArray("degindextab", indexArray, indexArray.size(), false);
	printArray("degtab", degArray, xmax, true);
    }
}
